@header pub const Vec2 = @Vector(2, f32)
@header pub const Mat4 = [4]@Vector(4, f32)
@ctype vec2 Vec2
@ctype mat4 Mat4

@vs vs
in vec2 mesh_xy;
in vec2 mesh_uv;

in vec2 instance_xbasis;
in vec2 instance_ybasis;
in vec2 instance_origin;
in vec4 instance_tint;
in float instance_depth;
in vec4 instance_rect;

uniform Projection {
	mat4 projection;
};

out vec4 color;
out vec2 uv;

void main() {
	// Calculate the world position of the vertex
	const vec2 world_position = instance_origin + mesh_xy.x * instance_xbasis + mesh_xy.y * instance_ybasis;
	const vec2 projected_position = (projection * vec4(world_position, 0, 1)).xy;

	// Set the position of the vertex in clip space
	gl_Position = vec4(projected_position, instance_depth, 1.0);
	color = instance_tint;

	// Calculate the width and height from left, top, right, bottom configuration
	const vec2 rect_pos = instance_rect.xy; // left, top
	const vec2 rect_size = instance_rect.zw - instance_rect.xy; // right - left, bottom - top

	// Calculate the adjusted UV coordinates using the instance_rect
	uv = rect_pos + (mesh_uv * rect_size);
}
@end

@fs fs
uniform texture2D tex;
uniform sampler smp;

in vec4 color;
in vec2 uv;

out vec4 texel;

void main() {
	texel = texture(sampler2D(tex, smp), uv) * color;

	if (texel.a == 0) {
		discard;
	}
}
@end

@program draw_2d vs fs
